//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with arts++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSPORTMATRIX_HH_
#define _ARTSPORTMATRIX_HH_

extern "C" {
#include <sys/time.h>
#include "caida_t.h"
}

#include "Arts.hh"

//---------------------------------------------------------------------------
//  class ArtsPortMatrix : public Arts
//---------------------------------------------------------------------------
//!  This class abstracts a port matrix.  A port matrix contains packet
//!  and byte counters for traffic sent from source transport port numbers
//!  to destination transport port numbers.  Note we just inherit from
//!  the Arts class and add some convenience functions for accessing
//!  the contained ArtsPortMatrixData object.
//---------------------------------------------------------------------------
class ArtsPortMatrix : public Arts
{
public:
  //-------------------------------------------------------------------------
  //                             ArtsPortMatrix()
  //.........................................................................
  //!  constructor
  //-------------------------------------------------------------------------
  ArtsPortMatrix();

  //-------------------------------------------------------------------------
  //                             ~ArtsPortMatrix()
  //.........................................................................
  //!  destructor
  //-------------------------------------------------------------------------
  ~ArtsPortMatrix();

  //-------------------------------------------------------------------------
  //                  inline uint16_t SampleInterval() const                 
  //.........................................................................
  //!  Returns the sampling ratio if the port matrix was built using
  //!  packet sampling techniques.  Normally this value is 1, meaning
  //!  we're working with complete data sets (for example, from Cisco
  //!  flow-export).
  //-------------------------------------------------------------------------
  inline uint16_t SampleInterval() const
  {
    return(this->PortMatrixData()->SampleInterval());
  }

  //-------------------------------------------------------------------------
  //         inline uint16_t SampleInterval(uint16_t sampleInterval)         
  //.........................................................................
  //!  Sets and returns the sampling ratio if the port matrix was built
  //!  using packet sampling techniques.  Normally this value is 1,
  //!  meaning we're working with complete data sets (for example, from
  //!  Cisco flow-export).
  //-------------------------------------------------------------------------
  inline uint16_t SampleInterval(uint16_t sampleInterval)
  {
    return(this->PortMatrixData()->SampleInterval(sampleInterval));
  }

  //-------------------------------------------------------------------------
  //                      inline uint32_t Count() const                      
  //.........................................................................
  //!  Returns the number of port matrix entries (cells) in an
  //!  ArtsPortMatrix object.
  //-------------------------------------------------------------------------
  inline uint32_t Count() const
  {
    return(this->PortMatrixData()->Count());
  }

  //-------------------------------------------------------------------------
  //                  inline uint32_t Count(uint32_t count)                  
  //.........................................................................
  //!  Sets and Returns the number of port matrix entries (cells) in an
  //!  ArtsPortMatrix object.
  //-------------------------------------------------------------------------
  inline uint32_t Count(uint32_t count)
  {
    return(this->PortMatrixData()->Count(count));
  }

  //-------------------------------------------------------------------------
  //                    inline uint64_t TotalPkts() const
  //.........................................................................
  //!  Returns the total packets of traffic in the ArtsPortMatrix object.
  //!  The total is the sum of all traffic from each source to each
  //!  destination.
  //-------------------------------------------------------------------------
  inline uint64_t TotalPkts() const
  {
    return(this->PortMatrixData()->TotalPkts());
  }

  //-------------------------------------------------------------------------
  //              inline uint64_t TotalPkts(uint64_t totalPkts)              
  //.........................................................................
  //!  Sets and returns the total packets of traffic in the
  //!  ArtsPortMatrixData object.  The total is the sum of all traffic
  //!  from each source to each destination.
  //-------------------------------------------------------------------------
  inline uint64_t TotalPkts(uint64_t totalPkts)
  {
    return(this->PortMatrixData()->TotalPkts(totalPkts));
  }
  
  //-------------------------------------------------------------------------
  //                    inline uint64_t TotalBytes() const
  //.........................................................................
  //!  Returns the total bytes of traffic in the ArtsPortMatrix object.
  //!  The total is the sum of all traffic from each source to each
  //!  destination.
  //-------------------------------------------------------------------------
  inline uint64_t TotalBytes() const
  {
    return(this->PortMatrixData()->TotalBytes());
  }

  //-------------------------------------------------------------------------
  //             inline uint64_t TotalBytes(uint64_t totalBytes)             
  //.........................................................................
  //!  Sets and returns the total bytes of traffic in the ArtsPortMatrix
  //!  object.  The total is the sum of all traffic from each source to
  //!  each destination.
  //-------------------------------------------------------------------------
  inline uint64_t TotalBytes(uint64_t totalBytes)
  {
    return(this->PortMatrixData()->TotalBytes(totalBytes));
  }

  //-------------------------------------------------------------------------
  //          inline vector<ArtsPortMatrixEntry> & PortEntries() const
  //.........................................................................
  //!  Return a reference to the vector of ArtsPortMatrixEntry objects.
  //!  Note that even though this member may be called from a const
  //!  object, the returned reference is not const; the vector is
  //!  mutable.
  //-------------------------------------------------------------------------
  inline vector<ArtsPortMatrixEntry> & PortEntries() const
  {
    return(this->PortMatrixData()->PortEntries());
  }

  //--------------------------------------------------------------------------
  //      friend istream & operator >> (istream & is, 
  //                                    ArtsPortMatrix & artsPortMatrix) 
  //..........................................................................
  //!  Reads an ArtsPortMatrix from an istream.  Note this is included
  //!  for use with an istream_iterator and is not intended to be used
  //!  in any other manner; it skips over other types of arts objects
  //!  in the input stream.
  //--------------------------------------------------------------------------
  friend istream & operator >> (istream & is,
                                ArtsPortMatrix & artsPortMatrix);
  
};


#endif  // _ARTSPORTMATRIX_HH_
